using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Webus.Index;
using Webus.Analysis;
using Webus.Search;
using Webus.Documents;

namespace FileSystemSearcher
{
    public partial class frmMain : Form
    {
        IIndexer indexer;
        ISearcher searcher;

        public frmMain()
        {
            InitializeComponent();

            indexer = new IndexManager(new MyAnalyzer());
            searcher = new IndexSearcher(indexer);
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
        }

        private void indexingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (txtKeyword.Enabled)
            {
                indexer.Close();
            }
            txtKeyword.Enabled = false;
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                string indexPath = string.Format(@"{0}\Index", Application.StartupPath);

                #region Create Index
                indexer.New(indexPath);
                foreach (string file in openFileDialog1.FileNames)
                {
                    using (StreamReader sr = new StreamReader(file))
                    {
                        Document doc = new Document();
                        doc.Fields.Add(new Field("FileName", file));
                        doc.Fields.Add(new Field("Content", sr.ReadToEnd(),
                            FieldAttributes.Analyse | FieldAttributes.Index | FieldAttributes.Compress));
                        indexer.Add(doc);
                    }
                }
                indexer.Close(); 
                #endregion

                #region Open Index
                indexer.Open(indexPath, IndexOpenMode.Read);
                #endregion

                txtKeyword.Enabled = true;
            }
        }

        private void txtKeyword_TextChanged(object sender, EventArgs e)
        {
            //ÿոָûĹؼ
            tvResult.Nodes.Clear();
            char[] keys = txtKeyword.Text.ToCharArray();
            //һ "key1 AND key2 AND key3 ..." ͵Query
            Query query = null;
            foreach (char key in keys)
            {
                Query q = new TermQuery(new Term("Content", key.ToString()));
                if (query == null)
                {
                    query = q;
                }
                else
                {
                    query &= q; //"&"Queryм㣬ΪһµQuery
                }
            }
            if (query == null) return;
            Hits hits = searcher.Search(query);//

            for (int i = 0; i < hits.Count; i++)
            {
                Document doc = hits.GetDoc(i);
                TreeNode node = tvResult.Nodes.Add(doc.GetField("FileName").Value.ToString());
                string content = doc.GetField("Content").Value.ToString();
                foreach (Position pos in hits[i].Positions)
                {
                    int h = content.LastIndexOf("\n", pos.Start);
                    int t = content.IndexOf("\n", pos.Start + pos.Length);
                    h = h >= 0 ? h : 0;
                    int l = t - h;
                    l = l >= 0 ? l : 0;
                    node.Nodes.Add(string.Format("[Position: {0}]   {1}", pos.Start, content.Substring(h, l).Trim()));
                }
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }

    class MyAnalyzer : IAnalyzer
    {
        public MyAnalyzer() { }
        public ITokenStream GetTokenStream(string value)
        {
            return new Webus.Analysis.TokenStreams.CharTokenStream(value);
        }

        public ITokenStream GetTokenStream(Field field)
        {
            return this.GetTokenStream(field.Value.ToString());
        }
    }
}